@@ -17,7 +17,15 @@ Some utility modules are also provided:
 
 from . import stream, pipe
 from .aiter_utils import async_, await_
-from .core import StreamEmpty, operator, streamcontext
+from .core import StreamEmpty, operator, pipable_operator, streamcontext
 
-__all__ = ['stream', 'pipe', 'async_', 'await_',
-           'operator', 'streamcontext', 'StreamEmpty', ]
+__all__ = [
+    "stream",
+    "pipe",
+    "async_",
+    "await_",
+    "operator",
+    "pipable_operator",
+    "streamcontext",
+    "StreamEmpty",
+]
