@@ -1,24 +1,26 @@
 import asyncio
-from aiostream import stream, pipe
+
+from aiostream import pipe, stream
 
 
-async def main():
+def square(x: int, *_: int) -> int:
+    return x**2
 
+
+async def main() -> None:
     # Create a counting stream with a 0.2 seconds interval
     xs = stream.count(interval=0.2)
 
     # Operators can be piped using '|'
-    ys = xs | pipe.map(lambda x: x ** 2)
+    ys = xs | pipe.map(square)
 
     # Streams can be sliced
     zs = ys[1:10:2]
 
     # Use a stream context for proper resource management
     async with zs.stream() as streamer:
-
         # Asynchronous iteration
         async for z in streamer:
-
             # Print 1, 9, 25, 49 and 81
             print("->", z)
 
@@ -36,6 +38,4 @@ async def main():
 
 
 # Run main coroutine
-loop = asyncio.get_event_loop()
-loop.run_until_complete(main())
-loop.close()
+asyncio.run(main())
