@@ -14,8 +14,10 @@ Test using netcat client:
     [...]
 """
 
+import math
 import asyncio
-from aiostream import stream, pipe
+
+from aiostream import pipe, stream
 
 # Constants
 
@@ -39,14 +41,24 @@ RESULT = """\
 # Client handler
 
 
-async def euclidean_norm_handler(reader, writer):
-
+async def euclidean_norm_handler(
+    reader: asyncio.StreamReader, writer: asyncio.StreamWriter
+) -> None:
     # Define lambdas
-    strip = lambda x: x.decode().strip()
-    nonempty = lambda x: x != ""
-    square = lambda x: x ** 2
-    write_cursor = lambda x: writer.write(b"> ")
-    square_root = lambda x: x ** 0.5
+    def strip(x: bytes, *_: object) -> str:
+        return x.decode().strip()
+
+    def nonempty(x: str) -> bool:
+        return x != ""
+
+    def square(x: float, *_: object) -> float:
+        return x**2
+
+    def write_cursor(_: float) -> None:
+        return writer.write(b"> ")
+
+    def square_root(x: float, *_: object) -> float:
+        return math.sqrt(x)
 
     # Create awaitable
     handle_request = (
@@ -58,7 +70,7 @@ async def euclidean_norm_handler(reader, writer):
         | pipe.map(square)
         | pipe.print("square: {:.2f}")
         | pipe.action(write_cursor)
-        | pipe.accumulate(initializer=0)
+        | pipe.accumulate(initializer=0.0)
         | pipe.map(square_root)
         | pipe.print("norm -> {:.2f}")
     )
@@ -77,27 +89,18 @@ async def euclidean_norm_handler(reader, writer):
 # Main function
 
 
-def run_server(bind="127.0.0.1", port=8888):
-
+async def main(bind: str = "127.0.0.1", port: int = 8888) -> None:
     # Start the server
-    loop = asyncio.get_event_loop()
-    coro = asyncio.start_server(euclidean_norm_handler, bind, port)
-    server = loop.run_until_complete(coro)
+    server = await asyncio.start_server(euclidean_norm_handler, bind, port)
 
     # Serve requests until Ctrl+C is pressed
     print("Serving on {}".format(server.sockets[0].getsockname()))
-    try:
-        loop.run_forever()
-    except KeyboardInterrupt:
-        pass
 
-    # Close the server
-    server.close()
-    loop.run_until_complete(server.wait_closed())
-    loop.close()
+    async with server:
+        await server.serve_forever()
 
 
 # Main execution
 
 if __name__ == "__main__":
-    run_server()
+    asyncio.run(main())
