@@ -9,7 +9,6 @@ event_loop
 
 @pytest.mark.asyncio
 async def test_streamcontext(event_loop):
-
     with event_loop.assert_cleanup():
         xs = stream.range(3) | add_resource.pipe(1)
         async with streamcontext(xs) as streamer:
@@ -28,7 +27,6 @@ async def test_streamcontext(event_loop):
 
 
 def test_operator_from_method():
-
     with pytest.raises(ValueError):
 
         class A:
@@ -43,7 +41,7 @@ def test_operator_from_method():
             async def method(cls, arg):
                 yield 1
 
-    with pytest.raises(AttributeError):
+    with pytest.raises(ValueError):
 
         class C:
             @operator
@@ -78,3 +76,39 @@ async def test_error_on_entering_a_stream(event_loop):
             assert False
 
     assert "Use the `stream` method" in str(ctx.value)
+
+
+def test_compatibility():
+    @operator
+    async def test1():
+        yield 1
+
+    with pytest.raises(AttributeError):
+        test1.pipe
+
+    match = "The `pipable` argument is deprecated."
+    with pytest.warns(DeprecationWarning, match=match):
+
+        @operator()
+        async def test2():
+            yield 1
+
+    with pytest.raises(AttributeError):
+        test2.pipe
+
+    with pytest.warns(DeprecationWarning, match=match):
+
+        @operator(pipable=False)
+        async def test3():
+            yield 1
+
+    with pytest.raises(AttributeError):
+        test3.pipe
+
+    with pytest.warns(DeprecationWarning, match=match):
+
+        @operator(pipable=True)
+        async def test4(source):
+            yield 1
+
+    test4.pipe
